import logging
from email.mime.application import MIMEApplication

from botocore.exceptions import ClientError
from django.conf import settings
from django.core.mail import EmailMessage

logger = logging.getLogger('dict_config_logger')
host = getattr(settings, 'SERVER_IP', False)


def send_notifications(email, sender, email_configuration):
    """This function sends email of a log file """

    logger.info('Sending email to recipients')
    # Replace sender@example.com with your "From" address.
    # This address must be verified with Amazon SES.
    SENDER = sender

    # Replace recipient@example.com with a "To" address. If your account
    # is still in the sandbox, this address must be verified.
    RECIPIENT = email

    # The subject line for the email.
    SUBJECT = email_configuration.get('Subject')

    # The full path to the file that will be attached to the email.
    # LOG_PATH = email_configuration.get('Log_path')
    ATTACHMENT = getattr(settings, "LOG_PATH", None)

    # The HTML body of the email.
    file_name = email_configuration.get('HTML_File')
    BODY_HTML = open('media/' + file_name).read().format(
        paragraph=email_configuration.get('Email_Content'),
        signature=email_configuration.get('Signature'),
        email_us=email_configuration.get('Email_Us'),
        faq_url=email_configuration.get('FAQ_URL'),
        unsubscribe=email_configuration.get('Unsubscribe_Email_ID'),
        banner=host + "/media/" + email_configuration.get('Banner'))
    # Define the attachment part and encode it using MIMEApplication.
    att = MIMEApplication(open(ATTACHMENT, 'rb').read())

    # Add a header to tell the email client to treat this part as an
    # attachment, and to give the attachment a name.
    att.add_header('Content-Disposition', 'attachment',
                   filename="OpenLXP notifications ")

    for each_recipient in RECIPIENT:
        try:
            # Provide the contents of the email.

            mail = EmailMessage(SUBJECT, BODY_HTML, SENDER,
                                [each_recipient])
            mail.content_subtype = "html"
            # Add the attachment to the parent container.
            mail.attach(att)
            mail.send()
            logging.FileHandler(getattr(settings, "LOG_PATH", None),
                                mode='w')
        # Display an error if something goes wrong.
        except ClientError as e:
            logger.error(e.response['Error']['Message'])
            continue


def send_notifications_with_msg(email, sender, msg, email_configuration):
    """This function sends email of a log file """

    logger.info('Sending email to recipients')
    # Replace sender@example.com with your "From" address.
    # This address must be verified with Amazon SES.
    SENDER = sender
    # Replace recipient@example.com with a "To" address. If your account
    # is still in the sandbox, this address must be verified.
    RECIPIENT = email
    # The subject line for the email.
    SUBJECT = email_configuration.get('Subject')

    # # The HTML body of the email.
    file_name = email_configuration.get('HTML_File')
    BODY_HTML = open('media/' + file_name).read().format(
        paragraph=msg,
        signature=email_configuration.get('Signature'),
        email_us=email_configuration.get('Email_Us'),
        faq_url=email_configuration.get('FAQ_URL'),
        unsubscribe=email_configuration.get('Unsubscribe_Email_ID'),
        banner=host + "/media/" + email_configuration.get('Banner'))

    for each_recipient in RECIPIENT:
        try:
            # Provide the contents of the email.

            mail = EmailMessage(SUBJECT, BODY_HTML, SENDER,
                                [each_recipient])
            mail.content_subtype = "html"
            # Add the attachment to the parent container.
            mail.send()
        # Display an error if something goes wrong.
        except ClientError as e:
            logger.error(e.response['Error']['Message'])
            continue



import logging

from django.core.management.base import BaseCommand

from openlxp_notifications.management.utils.notification import (
    send_notifications, send_notifications_with_msg)
from openlxp_notifications.models import (EmailConfiguration,
                                          ReceiverEmailConfiguration,
                                          SenderEmailConfiguration)

logger = logging.getLogger('dict_config_logger')


def get_sender_email():
    """Getting sender email id"""

    sender_email_configuration = SenderEmailConfiguration.objects.first()
    if sender_email_configuration:
        sender = sender_email_configuration.sender_email_address
    else:
        sender = None
    return sender


def send_log_email():
    """ function to send emails of log file to personas"""

    # getting email id to send email
    email_data = ReceiverEmailConfiguration.objects.values_list(
        'email_address', flat=True)
    email = list(email_data)
    # Getting sender email id
    sender = get_sender_email()

    if sender and email:
        email_configuration = EmailConfiguration.objects.filter(
            Content_Type='ATTACHMENT').values('HTML_File', 'Subject',
                                              'Email_Content', 'Signature',
                                              'Email_Us', 'FAQ_URL',
                                              'Unsubscribe_Email_ID',
                                              'Content_Type', 'Banner').first()
        if email_configuration:
            send_notifications(email, sender, email_configuration)
        else:
            logger.error("Email configuration for email format not set. "
                         "Email failed to send")
    else:
        logger.error("Sender/Receiver email does not exist. "
                     "Email failed to send")


def send_log_email_with_msg(email, msg):
    """ function to send emails of log file to personas"""
    # Getting sender email id
    sender = get_sender_email()

    if sender:
        email_configuration = EmailConfiguration.objects.filter(
            Content_Type='MESSAGE').values('HTML_File', 'Subject',
                                           'Email_Content', 'Signature',
                                           'Email_Us', 'FAQ_URL',
                                           'Unsubscribe_Email_ID',
                                           'Content_Type', 'Banner').first()
        if email_configuration:
            send_notifications_with_msg(email, sender,
                                        msg, email_configuration)
        else:
            logger.error("Email configuration for email format not set. "
                         "Email failed to send")
    else:
        logger.error("Sender email does not exist. Email failed to send")


class Command(BaseCommand):
    """Django command to send an emails to the filer/personas, when the log
    warning/error occurred in the metadata EVTVL process."""

    def handle(self, *args, **options):
        """Email log notification is sent to filer/personas when warning/error
        occurred in EVTVL process"""
        send_log_email()



# Generated by Django 3.2.3 on 2022-08-22 19:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openlxp_notifications', '0002_rename_logs_type_emailconfiguration_content_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailconfiguration',
            name='Banner',
            field=models.ImageField(default='banner.jpg', help_text='add logo image', upload_to='banner'),
        ),
    ]



import logging
from unittest.mock import mock_open, patch

from ddt import ddt
from django.test import tag

from openlxp_notifications.management.utils.notification import (
    send_notifications, send_notifications_with_msg)
from openlxp_notifications.management.utils.ses_client import \
    check_if_email_verified
from openlxp_notifications.models import EmailConfiguration

from .test_setup import TestSetUp

logger = logging.getLogger('dict_config_logger')


@tag('unit')
@ddt
class UtilsTests(TestSetUp):
    """Unit Test cases for utils """

    # Test cases for NOTIFICATION
    def test_check_if_email_verified(self):
        """Test to check if email id from user is verified """
        with patch('openlxp_notifications.management.utils.ses_client'
                   '.list_email_verified') as mock_list:
            mock_list.return_value = self.receive_email_list
            email_value = 'receiver1@openlxp.com'
            return_val = check_if_email_verified(email_value)
            self.assertFalse(return_val)

    def test_check_if_email_not_verified(self):
        """Test to check if email id from user is verified """
        with patch('openlxp_notifications.management.utils.ses_client'
                   '.list_email_verified') as mock_list:
            mock_list.return_value = self.receive_email_list
            email_value = 'receiver2@openlxp.com'
            return_val = check_if_email_verified(email_value)
            self.assertTrue(return_val)

    def test_send_notifications(self):
        """Test for the function  which sends email of a log file """
        with patch('openlxp_notifications.management.utils.notification'
                   '.EmailMessage') as mock_send, \
                patch('openlxp_notifications.management.utils.ses_client'
                      '.boto3.client'), \
                patch('openlxp_notifications.management.utils.notification'
                      '.send_notifications'), \
                patch(
                    'openlxp_notifications.management.utils.notification.'
                    'open', mock_open(read_data="data")):
            email_config = EmailConfiguration(
                Subject=self.Subject, Email_Content=self.Email_Content,
                Signature=self.Signature, Email_Us=self.Email_Us,
                FAQ_URL=self.FAQ_URL,
                Unsubscribe_Email_ID=self.Unsubscribe_Email_ID,
                Content_Type='ATTACHMENT', HTML_File='HTML_Files/My_Html.html')
            email_config.save()
            email_configuration = EmailConfiguration.objects.filter(
                Content_Type='ATTACHMENT').values('HTML_File', 'Subject',
                                                  'Email_Content',
                                                  'Signature', 'Email_Us',
                                                  'FAQ_URL',
                                                  'Unsubscribe_Email_ID',
                                                  'Content_Type'
                                                  ).first()
            send_notifications(self.receive_email_list, self.sender_email,
                               email_configuration)

            self.assertEqual(mock_send.call_count, 2)

    def test_send_notifications_with_msg(self):
        """Test for the function  which sends email of a message """
        with patch('openlxp_notifications.management.utils.notification'
                   '.EmailMessage') as mock_send, \
                patch('openlxp_notifications.management.utils.ses_client'
                      '.boto3.client'), \
                patch('openlxp_notifications.management.utils.notification'
                      '.send_notifications'), \
                patch(
                    'openlxp_notifications.management.utils.notification.'
                    'open', mock_open(read_data="data")):
            email_config = EmailConfiguration(
                Subject=self.Subject, Email_Content=self.Email_Content,
                Signature=self.Signature, Email_Us=self.Email_Us,
                FAQ_URL=self.FAQ_URL,
                Unsubscribe_Email_ID=self.Unsubscribe_Email_ID,
                Content_Type='MESSAGE', HTML_File='HTML_Files/My_Html.html')
            email_config.save()
            email_configuration = EmailConfiguration.objects.filter(
                Content_Type='MESSAGE').values('HTML_File', 'Subject',
                                               'Email_Content',
                                               'Signature', 'Email_Us',
                                               'FAQ_URL',
                                               'Unsubscribe_Email_ID',
                                               'Content_Type'
                                               ).first()
            send_notifications_with_msg(self.receive_email_list,
                                        self.sender_email, 'Message',
                                        email_configuration)

            self.assertEqual(mock_send.call_count, 2)
